@@ -1,5 +1,7 @@
 import os
 import requests
+import hashlib
+from typing import Optional, Union
 from lxml import etree
 
 _here = os.path.dirname(__file__)
@@ -14,15 +16,32 @@ class Validator:
         xmlschema_doc = etree.parse(self.get_schema(xsd_path))
         self.xmlschema = etree.XMLSchema(xmlschema_doc)
 
+    @staticmethod
+    def retrieve_xsd(file: Union[str, etree._ElementTree]) -> Optional[str]:
+        ns = '{http://www.w3.org/2001/XMLSchema-instance}'
+        if isinstance(file, str):
+            document = etree.parse(file)
+        else:
+            document = file
+        schemaLink = document.getroot().get(ns + 'schemaLocation')
+        if schemaLink:
+            for link in schemaLink.split():
+                if link.endswith(".xsd"):
+                    return Validator.get_schema(link)
+        return None
+
     @staticmethod
     def get_schema(xsd_path):
         if xsd_path.startswith("http://") or xsd_path.startswith("https://"):
+            new_xsd_path = f"downloaded_{hashlib.sha256(xsd_path.encode()).hexdigest()}.xsd"
+            if os.path.exists(new_xsd_path):
+                return new_xsd_path
             schema_req = requests.get(xsd_path)
             schema_req.raise_for_status()
-            new_path = "downloaded.xsd"
-            with open(new_path, "w") as f:
+            with open(new_xsd_path, "w") as f:
                 f.write(schema_req.text)
-            return new_path
+            return new_xsd_path
+
         elif xsd_path in Schemas:
             return Schemas[xsd_path]
         return xsd_path
